	INCLUDE EFR32MG12.inc			; CPU register definitions
	INCLUDE	i2c.inc	
	IMPORT	I2Cbuf	
	EXPORT	press
	EXPORT	temp	
	EXPORT	dig_T1	
		
BMP280_SLAVE_ADDRESS	EQU	0x77    	; sensor slave address

;-------------------------------------	; non-initialized DATA segment in RAM 
	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=3
dig_T1	SPACE	2				; calibration coefficients
dig_T2	SPACE	2
dig_T3	SPACE	2	
dig_P1	SPACE	2			
dig_P2	SPACE	2
dig_P3	SPACE	2
dig_P4	SPACE	2
dig_P5	SPACE	2
dig_P6	SPACE	2
dig_P7	SPACE	2
dig_P8	SPACE	2
dig_P9	SPACE	2
adc_P	SPACE	4				; raw sensor data
adc_T	SPACE	4	
press	SPACE	4				; compensated values
temp	SPACE	4
	
	
	AREA    |.text|, CODE, READONLY		; another section in the CODE area
BMP280_setup	PROC				; read calibration data	
	EXPORT	BMP280_setup
	push	{LR, R0-R3}			; save used regs
	mov	R0, #BMP280_SLAVE_ADDRESS
	ldr	R1, =I2Cbuf
	mov	R2, #2				; write 2 bytes
	mov	R3, #0xF5			; config reg address		
	strb	R3, [R1]			; load config reg address
	mov	R3, #0xB4			; t_sb=1000, filter=16
	strb	R3, [R1, #1]			; load config register
	bl	I2CMasterWriteBuf
	pop	{PC, R0-R3}
	ENDP
		
;-------------------------------------------------------------------------------		
BMP280_read_calib	PROC			; read calibration data	
	EXPORT	BMP280_read_calib
	push	{LR, R0-R3}			; save used regs
	mov	R0, #BMP280_SLAVE_ADDRESS
	ldr	R1, =dig_T1
	mov	R2, #24				; read 24 bytes
	mov	R3, #0x88			; start address
	bl	I2CMasterReadBuf
	pop	{PC, R0-R3}
	ENDP

;-------------------------------------------------------------------------------
BMP280_read_data	PROC			; read raw sensor data
	EXPORT	BMP280_read_data		
	push	{LR, R0-R3}			; save used regs
	mov	R0, #BMP280_SLAVE_ADDRESS
	ldr	R1, =adc_P
	mov	R2, #6				; read 6 bytes
	mov	R3, #0xF7			; start address
	bl	I2CMasterReadBuf
	
	ldr	R0, [R1, #3]			; read temp
	rev	R0, R0				; reverse bytes order
	lsr	R0, #8
	str	R0, [R1, #4]			; save raw temp data
	
	ldr	R0, [R1]			; load press
	rev	R0, R0				; reverse bytes order
	lsr	R0, #8
	str	R0, [R1]			; save raw press data
	pop	{PC, R0-R3}
	ENDP

;-------------------------------------------------------------------------------
BMP280_request_conversion	PROC		; request single conversion
	EXPORT	BMP280_request_conversion		
	push	{LR, R0-R3}			; save used regs
	mov	R0, #BMP280_SLAVE_ADDRESS
	ldr	R1, =I2Cbuf
	mov	R2, #2				; write 2 bytes
	mov	R3, #0xF4			; ctrl_meas reg address
	strb	R3, [R1]			; load ctrl_meas reg address
	mov	R3, #0x55			; osrs_t=2, osrs_p=16, mode=forced
	strb	R3, [R1, #1]			; load config register	
	bl	I2CMasterWriteBuf
	pop	{PC, R0-R3}
	ENDP	

;-------------------------------------------------------------------------------
BMP280_compute_values	PROC
	EXPORT	BMP280_compute_values
	push	{LR, R0-R5, R10-R12}
;-------COMPUTING TEMPERATURE--------------------            	
	ldr	R0, =adc_T			; load raw temp value
	ldr	R0, [R0]			; R0 = adc_T
	lsr	R0, #4				; get rid of 4 trailing zeros
	
	lsr	R3, R0, #3			; R3 = adc_T>>3
	ldr	R4, =dig_T1
	ldrh	R1, [R4]			; R1 = dig_T1
	sub	R5, R3, R1, LSL #1		; term
	ldrsh	R2, [R4, #2]			; R2 = dig_T2
	mul	R11, R2, R5
	asr	R11, #11			; R11 = var1
	
	lsr	R3, R0, #4			; R3 = adc_T>>4
	sub	R5, R3, R1
	mul	R5, R5
	asr	R5, #12				; R5 = term
	ldrsh	R3, [R4, #4]			; R3 = dig_T3
	mul	R12, R5, R3			
	asr	R12, #14			; R12 = var2
	
	add	R10, R11, R12			; R10 = t_fine
	add	R1, R10, R10, LSL #2		; R1 = t_fine*5
	add	R1, #128
	asr	R1, #8				; R1 = T
	
	ldr	R0, =temp
	str	R1, [R0]			; save temp in RAM

;-------COMPUTING PRESSURE-----------------------
	ldr	R0, =adc_P			; load raw press value
	ldr	R0, [R0]			; R0 = adc_P
	lsr	R0, #4				; get rid of 4 trailing zeros
	ldr	R4, =dig_P1

; PUT YOUR CODE HERE. Use t_fine value in R10. Load computed pressure in R10

	ldr	R0, =press
	mov	R10, #0				; DELETE THIS LINE AFTER YOU COMPUTE PRESSURE
	str	R10, [R0]			; save press in RAM
	pop	{PC, R0-R5, R10-R12}
	ENDP
		
	ALIGN
	END	