	INCLUDE EFR32MG12.inc			; CPU register definitions	

;-------------------------------------	; non-initialized DATA segment in RAM 
	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=3
I2Cbuf	SPACE	16	
	EXPORT	I2Cbuf
			
	AREA    |.text|, CODE, READONLY		; another section in the CODE area	
;-------------------------------------------------------------------------------
I2C1_start	PROC				; start I2C session
	EXPORT	I2C1_start
	PUTW	(1<<5)+1, I2Cn_CMD, I2C1	; generate START			
is1	ldr	R1, [R0, #I2Cn_IF]		; R1 = status
	ands	R1, #1				; wait for composing the signal
	beq	is1
	PUTW	1, I2Cn_IFC			; clear start flag		
	bx	LR
	ENDP

;------------------------------------------------------------------------------
I2C1_rep_start	PROC				; repeated start
	EXPORT	I2C1_rep_start
	PUTW	1, I2Cn_CMD, I2C1		; generate START			
is2	ldr	R1, [R0, #I2Cn_IF]		; R1 = status
	ands	R1, #1				; wait for composing the signal
	beq	is2
	PUTW	1, I2Cn_IFC			; clear start flag		
	bx	LR
	ENDP
		
;------------------------------------------------------------------------------
I2C1_stop	PROC				; stop I2C session
	EXPORT	I2C1_stop
	GETW	I2Cn_IF, I2C1	
	ands	R1, #(1<<3)			; wait for shift register empty	
	beq	I2C1_stop
	PUTW	1<<7, I2Cn_IFC
	PUTW	2, I2Cn_CMD			; generate STOP
is3
	GETW	I2Cn_IF, I2C1	
	ands	R1, #(1<<8)			; wait for the stop flag to set
	beq	is3
	PUTW	1<<8, I2Cn_IFC			; clear stop flag
	bx	LR
	ENDP
	
;------------------------------------------------------------------------------	
I2C1_write	PROC				; send byte R0 to slave
	EXPORT	I2C1_write
	ldr	R1, =I2C1
	strb	R0, [R1, #I2Cn_TXDATA]		; load byte to shift buffer
is4
	GETW	I2Cn_IF, I2C1
	tst	R1, #((1<<11)+(1<<7)+(1<<4))	; wait for the TXBL flag to set
	beq	is4
	
	tst	R1, #(1<<7)			; check for NACK flag
	beq	is4e
	PUTW	1<<7, I2Cn_IFC			; clear NACK flag
	PUTW	1<<4, I2Cn_CMD			; send continue command	
is4e	bx	LR	
	ENDP
		
;------------------------------------------------------------------------------
I2C1_read	PROC				; read byte from slave
	EXPORT	I2C1_read
	ldr	R1, =I2C1
	mov	R2, #(1<<2)			; send ACK
	str	R2, [R1, #I2Cn_CMD]	
is5	ldr	R0, [R1, #I2Cn_IF]	
	ands	R0, #(1<<5)			; wait for byte to receive	
	beq	is5	
	ldrb	R0, [R1, #I2Cn_RXDATA]		; R0 = received byte	
	bx	LR		
	ENDP

;------------------------------------------------------------------------------
I2C1_read_last	PROC				; read byte from slave
	EXPORT	I2C1_read_last
	ldr	R1, =I2C1
	mov	R2, #(1<<3)			; send NACK
	str	R2, [R1, #I2Cn_CMD]	
is6	ldr	R0, [R1, #I2Cn_IF]	
	ands	R0, #(1<<5)			; wait for byte to receive	
	beq	is6	
	ldrb	R0, [R1, #I2Cn_RXDATA]		; R0 = received byte	
	bx	LR		
	ENDP
		
;------------------------------------------------------------------------------
I2CMasterWriteBuf	PROC			; R0=slave address, R1=buffer, R2=length
	EXPORT	I2CMasterWriteBuf
	push	{LR, R0-R4}			; save used regs
	bl	I2C1_start			; generate START
	ldr	R0, [SP]		
	lsl	R0, #1
	bl	I2C1_write			; send slave address + W
		
	ldr	R3, [SP, #4]			; R3 = buffer ptr
	ldr	R4, [SP, #8]			; R4 = buffer length
is7	ldrb	R0, [R3], #1			; get byte to send	
	bl	I2C1_write			; send it out
	subs	R4, #1
	bne	is7

is8
	GETW	I2Cn_IF, I2C1
	tst	R1, #(1<<3)			; wait for the transfer end
	beq	is8
	PUTW	1<<3, I2Cn_IFC			; clear TXC flag
	bl	I2C1_stop			; generate STOP
	pop	{PC, R0-R4}			; restore regs and exit
	ENDP

;------------------------------------------------------------------------------
I2CMasterReadBuf	PROC			; R0=slave address, R1=&buffer, R2=length, R4=register
	EXPORT	I2CMasterReadBuf		; read slave data from register R4
	push	{LR, R0-R4}			; save used regs
	bl	I2C1_start			; generate START
	ldr	R0, [SP]		
	lsl	R0, #1
	bl	I2C1_write			; send slave address + W
	ldr	R0, [SP, #12]			; R0 = register address
	bl	I2C1_write			; send register number
	
is9
	GETW	I2Cn_IF, I2C1
	tst	R1, #(1<<3)			; wait for the transfer end
	beq	is9
	PUTW	1<<3, I2Cn_IFC			; clear TXC flag

	bl	I2C1_rep_start			; generate REP_START
	ldr	R0, [SP]		
	lsl	R0, #1
	add	R0, #1
	bl	I2C1_write			; send slave address + R

	ldr	R3, [SP, #4]			; R3 = buffer ptr
	ldr	R4, [SP, #8]			; R4 = buffer length
	sub	R4, #1
is10	bl	I2C1_read			; R0 = received byte
	strb	R0, [R3], #1			; load byte to buffer	
	subs	R4, #1
	bne	is10

	bl	I2C1_read_last			; get the last byte
	strb	R0, [R3]			; add it to buffer
	bl	I2C1_stop			; generate STOP
	pop	{PC, R0-R4}			; restore used regs
	ENDP

;------------------------------------------------------------------------------
I2CMasterReadBuf2	PROC			; R0=slave address, R1=&buffer, R2=length
	EXPORT	I2CMasterReadBuf2		; read slave data without writing
	push	{LR, R0-R4}			; save used regs
	bl	I2C1_start			; generate START
	ldr	R0, [SP]		
	lsl	R0, #1
	add	R0, #1
	bl	I2C1_write			; send slave address + R

is11
	GETW	I2Cn_IF, I2C1
	tst	R1, #(1<<3)			; wait for the transfer end
	beq	is11
	PUTW	1<<3, I2Cn_IFC			; clear TXC flag

	ldr	R3, [SP, #4]			; R3 = buffer ptr
	ldr	R4, [SP, #8]			; R4 = buffer length
	sub	R4, #1
is12	bl	I2C1_read			; R0 = received byte
	strb	R0, [R3], #1			; load byte to buffer	
	subs	R4, #1
	bne	is12

	bl	I2C1_read_last			; get the last byte
	strb	R0, [R3]			; add it to buffer
	bl	I2C1_stop			; generate STOP
	pop	{PC, R0-R4}			; restore used regs
	ENDP
		
	ALIGN	
	END